import structlog
from rest_framework import serializers

from activate_account.models import Account
from authentication.constants import SESSION_GRANT_TYPE
from authentication.errors import InvalidGrantError, UnsupportedGrantTypeError

logger = structlog.get_logger(__name__)


class TokenResponseSerializer(serializers.Serializer):
    """
    Defines the structure of a successful token response.
    """

    expires_in = serializers.IntegerField()
    access_token = serializers.CharField()
    token_type = serializers.CharField()


class TokenRequestSerializer(serializers.Serializer):
    """
    Represents the data required to request a token.
    """

    grant_type = serializers.CharField()
    crsid = serializers.CharField(required=False, allow_blank=True)
    last_name = serializers.CharField(required=False, allow_blank=True)
    date_of_birth = serializers.DateField(format="%Y-%m-%d")
    code = serializers.CharField()

    def validate_grant_type(self, grant_type):
        """
        Validates the grant type provided in the token request.

        This method checks if the provided grant type matches the SESSION_GRANT_TYPE. If not,
        it raises an UnsupportedGrantTypeError.

        Args:
            grant_type (str): The grant type specified in the request.

        Returns:
            str: The validated grant type if it matches the expected type.

        Raises:
            UnsupportedGrantTypeError: If the provided grant type is not supported.
        """

        if grant_type != SESSION_GRANT_TYPE:
            raise UnsupportedGrantTypeError("Unsupported grant type")

        return grant_type

    def validate(self, data):
        """
        Validates the entire data set of the token request.

        This method performs checks to ensure that exactly one of the identification fields
        (crsid or last name) is provided and that an account exists that matches the provided
        credentials including the 'date_of_birth' and 'code'. If the validations pass, the account
        object is included in the validated data returned.

        Args:
            data (dict): The complete data of the token request to be validated.

        Returns:
            dict: The original data dictionary with an added 'account' key that holds the Account
            object.

        Raises:
            serializers.ValidationError: If both or neither of the identification fields (crsid,
            last name) are provided.
            InvalidGrantError: If no matching account is found based on the provided credentials.
        """

        if "crsid" in data and "last_name" in data:
            raise serializers.ValidationError("At most one of crsid and last name may be provided")
        if "crsid" not in data and "last_name" not in data:
            raise serializers.ValidationError(
                "At least one of crsid and last name must be provided"
            )

        try:
            if "crsid" in data:
                account = Account.objects.get(
                    crsid=data["crsid"], date_of_birth=data["date_of_birth"], code=data["code"]
                )
            else:
                account = Account.objects.get(
                    last_name=data["last_name"],
                    date_of_birth=data["date_of_birth"],
                    code=data["code"],
                )
        except Account.DoesNotExist:
            logger.warn(
                "No matching user in token request.",
                crsid=data.get("crsid"),
                last_name=data.get("last_name"),
                date_of_birth=data.get("date_of_birth"),
            )
            raise InvalidGrantError("No matching account")

        # Set account in validated data
        data["account"] = account

        return data


class TokenErrorSerializer(serializers.Serializer):
    """
    OAuth2-compatible error response returned from the token endpoint when a token could not be
    issued.
    """

    error = serializers.ChoiceField(
        choices=[
            (
                "unsupported_grant_type",
                "The authorization grant type is not supported by " "the authorization server.",
            ),
            (
                "invalid_request",
                "The request is missing a required parameter, includes "
                "an unsupported parameter value or is otherwise malformed.",
            ),
            ("invalid_grant", "The provided credentials do not match any known user"),
        ]
    )
    error_description = serializers.CharField(required=False)
    error_uri = serializers.URLField(required=False)


class EmptySerializer(serializers.Serializer):
    """
    Defines an empty response.
    """

    pass
